if not IsLoggedIn then
	-- wow 2.0
	return
end
local sm = AceLibrary("SharedMedia-1.0")
local L = AceLibrary("AceLocale-2.2"):new("Quartz")

local QuartzBuff = Quartz:NewModule('Buff')
local self = QuartzBuff
local Quartz = Quartz

local GetTime = GetTime
local table_sort = table.sort

local db

local new, del
do
	local cache = setmetatable({}, {__mode='k'})
	function new()
		local t = next(cache)
		if t then
			cache[t] = nil
			return t
		else
			return {}
		end
	end
	function del(t)
		for k in pairs(t) do
			t[k] = nil
		end
		cache[t] = true
		return nil
	end
end
local OnUpdate
do
	local min = L["%dm"]
	local function timenum(num)
		if num <= 10 then
			return ('%.1f'):format(num)
		elseif num <= 60 then
			return ('%d'):format(num)
		else
			return min:format(num / 60)
		end
	end
	function OnUpdate(frame)
		local currentTime = GetTime()
		local endTime = frame.endTime
		if currentTime > endTime then
			self:UpdateBars()
		else
			local remaining = (currentTime - frame.startTime)
			frame:SetValue(endTime - remaining)
			frame.timetext:SetText(timenum(endTime - currentTime))
		end
		
	end
end
local function OnShow(frame)
	frame:SetScript('OnUpdate', OnUpdate)
end
local function OnHide(frame)
	frame:SetScript('OnUpdate', nil)
end

local bars = setmetatable({}, {
	__index = function(t,k)
		local bar = CreateFrame('StatusBar', nil, UIParent)
		t[k] = bar
		bar:SetFrameStrata('MEDIUM')
		bar:Hide()
		bar:SetScript('OnShow', OnShow)
		bar:SetScript('OnHide', OnHide)
		bar:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16})
		bar:SetBackdropColor(0,0,0)
		bar.text = bar:CreateFontString(nil, 'OVERLAY')
		bar.timetext = bar:CreateFontString(nil, 'OVERLAY')
		bar.icon = bar:CreateTexture(nil, 'DIALOG')
		
		Quartz.ApplySettings()
		return bar
	end
})

function QuartzBuff:OnInitialize()
	db = Quartz:AcquireDBNamespace("Buff")
	Quartz:RegisterDefaults("Buff", "profile", {
		bufftexture = 'LiteStep',
		buffwidth = 120,
		buffheight = 12,
		bufffont = 'Friz Quadrata TT',
		bufffontsize = 9,
		buffalpha = 1,
		
		buffcolor = {0,0.49, 1},
		debuffcolor = {1.0,0.7, 0},
		
		bufftextcolor = {1,1,1},
	})
end
function QuartzBuff:OnEnable()
	self:RegisterBucketEvent("UNIT_AURA", 0.5)
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "UpdateBars")
	Quartz.ApplySettings()
end
function QuartzBuff:OnDisable()
	for _, v in pairs(bars) do
		v:Hide()
	end
end
function QuartzBuff:UNIT_AURA(units)
	for unit in pairs(units) do
		if unit == 'target' then
			self:UpdateBars()
		end
	end
end
do
	local tmp = {}
	local function sort(a,b)
		if a.isbuff == b.isbuff then
			return a.name < b.name
		else
			return a.isbuff
		end
	end
	function QuartzBuff:UpdateBars()
		local currentTime = GetTime()
		for k in pairs(tmp) do
			tmp[k] = del(tmp[k])
		end
		for i = 1, 32 do
			local name, _, texture, _, duration, remaining = UnitBuff('target', i)
			if not name then
				break
			end
			if duration then
				local t = new()
				tmp[#tmp+1] = t
				t.name = name
				t.texture = texture
				t.duration = duration
				t.remaining = remaining
				t.isbuff = true
			end
		end
		for i = 1, 40 do
			local name, _, texture, _, _, duration, remaining = UnitDebuff('target', i)
			if not name then
				break
			end
			if duration then
				local t = new()
				tmp[#tmp+1] = t
				t.name = name
				t.texture = texture
				t.duration = duration
				t.remaining = remaining
			end
		end
		table_sort(tmp, sort)
		local maxindex = 0
		for k,v in ipairs(tmp) do
			maxindex = k
			local bar = bars[k]
			bar.text:SetText(v.name)
			bar.icon:SetTexture(v.texture)
			local elapsed = (v.duration - v.remaining)
			local startTime, endTime = (currentTime - elapsed), (currentTime + v.remaining)
			bar.startTime = startTime
			bar.endTime = endTime
			bar:SetMinMaxValues(startTime, endTime)
			bar:Show()
			if v.isbuff then
				bar:SetStatusBarColor(unpack(db.profile.buffcolor))
			else
				bar:SetStatusBarColor(unpack(db.profile.debuffcolor))
			end
		end
		for i = maxindex+1, #bars do
			bars[i]:Hide()
		end
	end
end
function QuartzBuff:ApplySettings()
	if Quartz:IsModuleActive('Buff') then
		local db = db.profile
		for i, v in pairs(bars) do
			v:ClearAllPoints()
			v:SetStatusBarTexture(sm:Fetch('statusbar', db.bufftexture))
			v:SetWidth(db.buffwidth)
			v:SetHeight(db.buffheight)
			v:SetScale(Quartz.db.profile.scale)
			v:SetAlpha(db.buffalpha)
			if i == 1 then
				v:SetPoint('BOTTOMRIGHT', QuartzCastBar, 'TOPRIGHT', -3, 1)
			else
				v:SetPoint('BOTTOMRIGHT', bars[i-1], 'TOPRIGHT', 0, 1)
			end
			
			local text = v.text
			text:ClearAllPoints()
			text:SetFont(sm:Fetch('font', db.bufffont), db.bufffontsize)
			text:SetShadowColor( 0, 0, 0, 1)
			text:SetShadowOffset( 0.8, -0.8 )
			text:SetTextColor(unpack(db.bufftextcolor))
			text:SetNonSpaceWrap(false)
			text:ClearAllPoints()
			text:SetWidth(db.buffwidth)
			text:SetPoint("LEFT", v, "LEFT", 2, 0)
			text:SetJustifyH("LEFT")
			
			local timetext = v.timetext
			timetext:ClearAllPoints()
			timetext:SetFont(sm:Fetch('font', db.bufffont), db.bufffontsize)
			timetext:SetShadowColor( 0, 0, 0, 1)
			timetext:SetShadowOffset( 0.8, -0.8 )
			timetext:SetTextColor(unpack(db.bufftextcolor))
			timetext:SetNonSpaceWrap(false)
			timetext:ClearAllPoints()
			timetext:SetWidth(db.buffwidth)
			timetext:SetPoint("RIGHT", v, "RIGHT", -2, 0)
			timetext:SetJustifyH("RIGHT")
			
			local icon = v.icon
			icon:SetWidth(db.buffheight)
			icon:SetHeight(db.buffheight)
			icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
			icon:ClearAllPoints()
			icon:SetPoint('RIGHT', v, "LEFT", -2, 0)
		end
		self:UpdateBars()
	end
end
do
	local function set(field, value)
		db.profile[field] = value
		Quartz.ApplySettings()
	end
	local function get(field)
		return db.profile[field]
	end
	local function setcolor(field, ...)
		db.profile[field] = {...}
		Quartz.ApplySettings()
	end
	local function getcolor(field)
		return unpack(db.profile[field])
	end
	Quartz.options.args.modules.args.buffs = {
		type = 'group',
		name = L["Buffs"],
		desc = L["Buffs"],
		args = {
			toggle = {
				type = 'toggle',
				name = L["Enable"],
				desc = L["Enable"],
				get = function()
					return Quartz:IsModuleActive('Buff')
				end,
				set = function(v)
					Quartz:ToggleModuleActive('Buff', v)
				end,
				order = 100,
			},
			bufftexture = {
				type = 'text',
				name = L["Texture"],
				desc = L["Set the Bar Texture"],
				validate = sm:List('statusbar'),
				order = 101,
				get = get,
				set = set,
				passValue = 'bufftexture',
			},
			buffwidth = {
				type = 'range',
				name = L["Buff Bar Width"],
				desc = L["Buff Bar Width"],
				min = 50,
				max = 300,
				step = 1,
				get = get,
				set = set,
				passValue = 'buffwidth',
				order = 102,
			},
			buffheight = {
				type = 'range',
				name = L["Buff Bar Height"],
				desc = L["Buff Bar Height"],
				min = 4,
				max = 25,
				step = 1,
				get = get,
				set = set,
				passValue = 'buffheight',
				order = 103,
			},
			bufffont = {
				type = 'text',
				name = L["Font"],
				desc = L["Font"],
				validate = sm:List('font'),
				get = get,
				set = set,
				passValue = 'bufffont',
				order = 104,
			},
			bufffontsize = {
				type = 'range',
				name = L["Font Size"],
				desc = L["Font Size"],
				min = 3,
				max = 15,
				step = 1,
				get = get,
				set = set,
				passValue = 'bufffontsize',
				order = 105,
			},
			buffalpha = {
				type = 'range',
				name = L["Alpha"],
				desc = L["Alpha"],
				min = 0.05,
				max = 1,
				step = 0.05,
				isPercent = true,
				get = get,
				set = set,
				passValue = 'buffalpha',
				order = 106,
			},
			buffcolor = {
				type = 'color',
				name = L["Buff Color"],
				desc = L["Buff Color"],
				get = getcolor,
				set = setcolor,
				passValue = 'buffcolor',
				order = 107,
			},
			debuffcolor = {
				type = 'color',
				name = L["Debuff Color"],
				desc = L["Debuff Color"],
				get = getcolor,
				set = setcolor,
				passValue = 'debuffcolor',
				order = 108,
			},
			bufftextcolor = {
				type = 'color',
				name = L["Text Color"],
				desc = L["Text Color"],
				get = getcolor,
				set = setcolor,
				passValue = 'bufftextcolor',
				order = 109,
			},
		},
	}
end